/* MultiLabel.h
 *
 * Declares the multilabel class.
 * This widget shows a number of labels on a bar with separators
 * between them. The separators can be moved by dragging them with 
 * the mouse.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef MULTILABEL_H
#define MULTILABEL_H

#include <qframe.h>
#include <qlist.h>
#include <qrect.h>
#include <qpainter.h>
#include <stdio.h>
#include <stdlib.h>
#include "MLItem.h"

/* The number of pixels on either side of the separator that will
   react to the mouse events */
#define SEP_WIDTH    5

class MultiLabel : public QFrame
{
    Q_OBJECT
public:
    MultiLabel (
	QWidget *parent = 0, 
	const char *name = 0, 
	WFlags f = 0);

    ~MultiLabel();

    void insertColumn (int index = -1);
    void insertColumn (const char *text, int index = -1);
    
    void deleteColumn (int index);

    void setColumnLabel (const char *text, int index);
    void setColumnWidth (int width, int index);
    void setColumnAlignment (int alignment, int index);
    void setColumnMargin (int margin, int index);
    void setColumnVisible (bool visible, int index);
    void setColumnFont (const QFont &f, int index);
    
    void setWidth (int width);
    void setAlignment (int alignment);
    void setMargin (int margin);
    void setAutoResize (bool enable);
    void setAllFonts (QFont f);
    void setDoDrag (bool enable);
    void setDrawLast (bool enable) { doLast = enable; }

    const char *columnLabel (int index);
    int columnWidth (int index);
    int columnAlignment (int index);
    int columnMargin (int index);
    QFont columnFont (int index);
    bool columnVisible (int index);
    bool autoResize () { return autoresize; }
    bool doDrag ();
    bool drawLast ()     { return doLast; }

public slots:
    void scroll (int pixels);
    void setScrollOffset (int pixels);
    
signals:
    void pressed (int index, int button);
    void released (int index);
    void clicked (int index, int button);
    void sepDblClicked (int index);
    void dblClicked (int index);
    void widthChanged (int newWidth, int index);
    void indexChanged (int oldIndex, int newIndex);
    void scrollChanged (int offset);

protected:
    void drawContents (QPainter *);

private:
    void updateLabels ();
    void adjustWidthAndSize ();
    int itemContains (int x);
    void insertItem (MultiLabelItem *ml, int index);
    int closestLeft (int x);
    void mousePressEvent (QMouseEvent *);
    void mouseReleaseEvent (QMouseEvent *);
    void mouseMoveEvent (QMouseEvent *);
    void mouseDoubleClickEvent (QMouseEvent *);

    QList<MultiLabelItem> items;
    bool autoresize;
    int scrollOffset;
    int lastPressedItem;
    int lastPressedSep;
    int lastStartX;
    int lastPressedX, lastPressedY;
    int oldLeft;
    bool doErase;
    bool drag;
    bool doLast;
};

#endif // MULTILABEL_H

