/* Shortcut.h
 *
 * Header file for shortcut.cc.
 * Declares the shortcut class, which is responsible for mapping 
 * shortcut keys to functions. The shortcuts may be several keypresses.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef SHORTCUT_H
#define SHORTCUT_H

#define SC_MATCH             0
#define SC_PARTIAL           1
#define SC_UNDEFINED         2

#define SC_OK                3
#define OVERRIDE             4
#define UNDERRIDE            5

#define MAXKEYS              20
#define CANCEL_TIME_OUT      500

#include <qobject.h>
#include <qkeycode.h>
#include <qtimer.h>
#include <qlist.h>
#include <qsignal.h>
#include <stdio.h>
#include <ctype.h>

struct ShortcutItem
{
    ShortcutItem (int *, int, int);
    ~ShortcutItem () { delete keys; delete signal; }
    int *keys;
    int n;
    int id;
    bool enabled;
    QSignal *signal;
};

class Shortcut : public QObject
{
    Q_OBJECT
public:
    Shortcut (QWidget *parent, const char *name = 0);
    ~Shortcut ();

    void setCancelKey (int key);
    bool insertItem (int keys[], int n, int id, bool override = FALSE);
    bool changeItem (int keys[], int n, int id, bool override = FALSE);
    
    void clear ();
    void startRecording ();
    int stopRecording (int *keys, int maxkeys);

    static QString keyToString (int key);
    
    bool isItemEnabled (int id);
    void setItemEnabled (int id, bool enable);

    bool connectItem (int id, 
		      const QObject *receiver, 
		      const char *member);
    bool disconnectItem (int id,
			 const QObject *receiver, 
			 const char *member);
    void print (FILE *fp);

signals:
    void activated (int id);
    void bufferChanged (const char *);

protected:
    bool eventFilter (QObject *, QEvent *);

private slots:
    void tlwDestroyed ();
    void clearBuffer ();

private:
    void addKey (int key);
    void cancelBuffer ();
    int compareItems (ShortcutItem *i1, ShortcutItem *i2);
    ShortcutItem *findItem (int id);
    int checkSequence ();
    int compareItem (ShortcutItem *i);
    bool identical (ShortcutItem *i1, ShortcutItem *i2);

    QWidget *tlw;		// Top level widget
    QString  scBuffer;
    int curKeys[MAXKEYS];
    int nextPos;
    int cancelKey;
    QTimer *timer;
    QList<ShortcutItem> *items;
    bool isTiming;
    bool isRecording;
};

extern int stringToKey (char* string, char** newString);

#endif
