/*
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef BARRACUDA_H
#define BARRACUDA_H

#include <qstring.h>
#include <qfont.h>
#include <keywords.h>
#include <stdio.h>
#include <options.h>

#define PARSER_ERROR_LEN 100

typedef enum {
  MACRO, STRING, UNDEFINED, CATENATED
} text_type_t;

class MacroOrString
{
  bool representation;		/* 0 for tex, 1 for converted */
  QString str;			/* Text or macro name */
public:
  static bool convertTex;
  static void setConvertTex (bool b);
  MacroOrString () { type = UNDEFINED; }
  void setStr (char* s);
  text_type_t type;
  const char* getString ();
  const char* getMacro ();
};

typedef enum { REDEFINITION, INVALID_FIELD, OK } set_field_t;

class Reference
{
  int type;			/* Reference type */
  MacroOrString fields[LAST_FIELD-FIRST_FIELD+1];

public:
  Reference(int _type) { type = _type; }
  QString id;			/* Key */
  QString otherAttributes;
  void addOtherAttribute (const char* id, const char* value, text_type_t);
  set_field_t setField (int f_type, char* str, text_type_t t, bool force);
  int hasField (int f_type);	/* Has field type? */
  int getType ();		/* Reference type */
  void setType (int r_type);	/* Set reference type */
  MacroOrString* getField (int f_type); /* Get field */
  void print (FILE* file);
  QString print ();
};

#define barracuda_error fprintf (stderr, "Internal error in source file %s on line %d\n", __FILE__, __LINE__), abort

/* Macro functions */
extern const char* lookupMacro (const char* str);
extern void printAllMacros (FILE* outFile);
extern void printMacrosForPrint (FILE* outFile);
extern void initMacros ();
extern bool enterMacro (const char* macroName, char* macroText, text_type_t);
extern int loadOptionsFromFile (char* filename, Options& ops);
extern int saveOptionsToFile (char* filename, Options& ops);

extern bool changeMacro (char* macroName, char* macroText, text_type_t t);
extern void discardMacros ();
void removeMacro (char* identifier);
extern void convertToCurrentFormat (QString& str, bool& representation);
extern bool noMacros ();
extern void unmarkAllMacros (void);

extern bool parseShortcutString (char* string, int line, shortcut* s);

typedef enum {TEX, PS, PRINTER} output_type_t;

extern bool print (bool printMacros, bool printReferences,
		   output_type_t outputType,
		   char* fileOrPrinterName, char* bibfilename, char* caption);

extern bool helpTheUser (void);
extern QString getHelpUrl ();

extern QString texToString (const char* str);
extern QString stringToTex (const char* str);
extern void escapeSpecialSymbols (QString& str, const char c);

extern QString externalFontRepresentation (QFont font);
extern bool internalFontRepresentation (QFont& font, const char* str);

#endif
