/*
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef DB_H
#define DB_H

#include <qstring.h>
#include <qlabel.h>
#include <Sort.h>

typedef enum {NO_DIRECTION, FORWARD, BACKWARD} searchDir;


class Database
{
public:
  Database(long initSize = 4711);
  ~Database();
  
  //Mutators 
  void sort(int (*)(Reference **, Reference **));
  void sort(SortSpec *, bool expanded);
  int readFile(const char *s);
  int saveFile(const char *s, bool);
  int  addReference(Reference *);
  void removeReference(long index, bool deleteRef = FALSE);
  void removeReference(Reference *r, bool deleteRef = FALSE);
  void removeAll(bool deleteRef = FALSE);
  int  merge(Database &b);
  void replaceRef(long index, Reference *newRef);
  void changed();

  //Inspectors
  long maxIndex();
  bool isEmpty();
  bool isSaved();
  long idCollision(QString s);
  Reference* operator[](long index);
  Database* select(int (*pred) (void *));
  Database* select(Reference *regExpRef, bool *,  
		   bool expanded, bool caseSense);
  long search(QString s, long startindex, searchDir dir, 
	      bool expanded, bool caseSense);   
  
private:
  long maxLength;
  long elements;
  Reference** V;
  bool saveFlag;
  int add_Reference_(Reference *r);
};

int readFromFile(char *filename, Database &b);
int saveToFile(char *filename, Database &b, bool);



#endif
