/*
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef __DEFS_H__

/* type definitions for various types desribed in 
   Fisher & LeBlanc's book ``Crafting a Compiler''.  */

#include <sparc_types.h>
#include <stddef.h>

/* --- only one of these two should be defined */ 
#define SINGLE_SCOPE_SYMBOL_TABLE 1
#undef MULTIPLE_SCOPE_SYMBOL_TABLE

/* --- inline directive --- */
#ifdef __GNUC__
#define INLINE __inline__
#elif defined(__cplusplus)
#define INLINE inline
#else
#define INLINE  
#endif


/* --- boolean type --- */
#ifdef __cplusplus
/* C++ has a built-in boolean type */
#define bool_t bool
#define FALSE false
#define TRUE true
#else
/* boolean type in C */
typedef enum bool_ { FALSE, TRUE } bool_t;
#endif /* __cplusplus */

/* --- simple typedefs --- */
typedef char *string_t;
#define STRING_T_TO_CHAR_PTR(s) ((char *) (s))

/* string table entry */
typedef struct string_entry_
{
  string_t ptr;
  unsigned int length;
  unsigned int hash_value;
  int ret_value;		/* IDENTIFIER, RW_ABS,... */
  struct string_entry_* next;
} string_entry_t;

/* symbol table entry */
typedef struct id_entry_
{
  struct string_entry_ *name;	/* pointer to string table */
#ifdef MULTIPLE_SCOPE_SYMBOL_TABLE
  int level;			/* scope level */
#endif
  struct attributes_ *attr;	/* attribute info */
  struct id_entry_ *next_symbol; /* used to link all symbols together */
  struct id_entry_ *next_chain;	/* link in hash table chain */
  struct id_entry_ *next_same_name; /* link to next entry with same symbol */
  /* coordinate_t src; */       /* source file coordinates */
} id_entry_t;

typedef struct symbol_table_
{
  size_t size;			/* hash table size minus 1 */
  size_t n_entries;		/* number of entries in the table */
  float threshold;		/* THRESHOLD > 0.0 specifies how full
				   the hash table can get before it
				   must grow. */
  id_entry_t **buckets;		/* pointer to vector of size SIZE + 1 */
  struct symbol_table_ *previous; /* pointer to previous table */
  id_entry_t *all;		/* used to chain all symbols together */
#ifndef MULTIPLE_SCOPE_SYMBOL_TABLE
  int level;			/* scope level corresponding to this table */
#endif
} symbol_table_t;

/* --- attributes --- */

typedef enum id_class_
{
  CONST, VARIABLE, ENUMCONST, FIELD, TYPENAME, PACKAGENAME, SUBPROGRAMNAME,
  BUILTINFUNCTION
} id_class_t;

typedef enum
{
  LOCAL, REGISTER, GLOBAL
} storage_class_t;

typedef struct address_
{
  bool_t indirect;
  bool_t read_only;
  storage_class_t class;
  address_range_t var_offset;	/* stack */
  string_entry_t* entry;	/* global (name) */
  int register_num;		/* (virtual) register */
  int label;
} address_t;

typedef struct field_
{
  bool_t is_tag;
  address_range_t field_offset;
  struct attributes_ *next;
  struct variant_des_ *enclosing_variant;
} field_t;

typedef struct enum_const_
{
  integer_t enum_value;
  struct attributes_ *next;
} enum_const_t;

typedef struct builtin_function_
{
  struct type_des_* return_type;
  value_kind_t operand1_type;
  value_kind_t operand2_type;
  builtin_op_t operation;
} builtin_function_t;

typedef struct type_ref_
{
  bool_t complete;
  bool_t private;
  struct type_des_ *object_type;
} type_ref_t;

typedef struct attributes_
{
  string_t str;
} attributes_t;

#define barracuda_error fprintf (stderr, "Internal error in source file %s on line %d\n", __FILE__, __LINE__), abort

#define __DEFS_H__
#endif /* __DEFS_H__ */
