/*
 * Reference class.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include <barracuda.h>
#include <qstring.h>

/* setField
 *
 * Args: type, value
 * => status
 */
set_field_t Reference::setField (int type, char* str, 
				 text_type_t t, bool force)
{
  /* Check that it's a valid field */
  if (!hasField (type))
    return INVALID_FIELD;

  /* Check for redefinition */
  if (!force)
    if (fields[type-FIRST_FIELD].getString ())
      return REDEFINITION;

  /* Set new value */
  fields[type-FIRST_FIELD].type = t;
  fields[type-FIRST_FIELD].setStr (str);
  return OK;
}

/* hasField
 *
 * Args: type
 * => 1 if required, 2 if optional, 0 if not valid
 */
int Reference::hasField (int f_type)
{
  static const int fields[LAST_REFERENCE-FIRST_REFERENCE+2]
    [LAST_FIELD-FIRST_FIELD+1] =
  { {1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1, 1, 0, 2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0},
    {1, 1, 0, 2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 0, 2, 2, 2, 2, 2, 0, 0, 0},
    {1, 1, 0, 1, 2, 2, 2, 2, 2, 2, 2, 0, 0, 2, 2, 0, 1, 2, 1, 2, 2, 0, 0, 0},
    {0, 1, 0, 1, 2, 2, 2, 2, 0, 2, 2, 0, 0, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0},
    {1, 1, 0, 1, 2, 2, 2, 2, 0, 2, 2, 0, 0, 2, 2, 0, 1, 2, 0, 0, 2, 0, 0, 0},
    {2, 1, 0, 2, 2, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 2, 0, 0},
    {1, 1, 0, 1, 2, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 0, 0, 2, 0, 2, 0, 0, 1, 0},
    {1, 1, 0, 1, 2, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 0, 0, 2, 0, 2, 0, 0, 1, 0},
    {1, 1, 0, 1, 2, 0, 2, 2, 0, 2, 2, 0, 0, 0, 0, 0, 0, 2, 0, 2, 0, 0, 0, 1},
    {2, 1, 0, 2, 2, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 2, 0, 2, 0, 0, 2, 0, 0, 0},
    {1, 1, 0, 2, 2, 0, 0, 2, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2, 2, 0, 2, 2, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0},
    {2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2}};

  if (f_type < FIRST_FIELD || f_type > LAST_FIELD)
    return 0;

  return fields[type-FIRST_REFERENCE][f_type-FIRST_FIELD];
}

/* getType
 *
 * => type of reference
 */
int Reference::getType ()
{
  return type;
}

/* setType
 *
 * Args: new type of reference
 */
void Reference::setType (int r_type)
{
  type = r_type;
}

/* getField
 *
 * Args: field to get
 * => value of field
 */
MacroOrString* Reference::getField (int f_type)
{
  /* Check that it's a valid field */
  if (!hasField (f_type))
    return 0;

  return &(fields[f_type-FIRST_FIELD]);
}

QString Reference::print ()
{
  QString ret, tmp;
  int i;

  ret.sprintf ("@%s{%s", keywords[type], (const char*) id);
  
  for (i = FIRST_FIELD; i <= LAST_FIELD; i++)
    if (hasField (i) && getField (i)->getMacro ())
      switch (getField (i)->type)
	{
	case STRING:
	  ret += tmp.sprintf (",\n  %s = \"%s\"", keywords[i],
			      getField (i)->getMacro ());
	  break;
	case CATENATED:
	  ret += tmp.sprintf (",\n  %s = %s", keywords[i],
			      getField (i)->getMacro ());
	  break;
	case MACRO:
	  ret += tmp.sprintf (",\n  %s = %s", keywords[i],
			      getField (i)->getMacro ());
	case UNDEFINED:
	  break;		// Ignore undefined fields
	default:
	  barracuda_error ();
	};

  if (!otherAttributes.isEmpty ())
    ret += otherAttributes;

  ret += tmp.sprintf ("}\n");

  return ret;
}

void Reference::print (FILE* file)
{
  int i;

  fprintf (file, "@%s{%s", keywords[type], (const char*) id);
  
  for (i = FIRST_FIELD; i <= LAST_FIELD; i++)
    if (hasField (i) && getField (i)->getMacro ())
      switch (getField (i)->type)
	{
	case STRING:
	  fprintf (file, ",\n  %s = \"%s\"", keywords[i],
		   getField (i)->getMacro ());
	  break;
	case CATENATED:
	  fprintf (file, ",\n  %s = %s", keywords[i],
		   getField (i)->getMacro ());
	  break;
	case MACRO:
	  fprintf (file, ",\n  %s = %s", keywords[i],
		   getField (i)->getMacro ());
	case UNDEFINED:
	  break;		// Ignore undefined fields
	default:
	  barracuda_error ();
	};

  if (!otherAttributes.isEmpty ())
    fprintf (file, "%s", (const char*) otherAttributes);

  fprintf (file, "}\n");
}

void Reference::addOtherAttribute (const char* id, const char* value, 
				   text_type_t type)
{
  otherAttributes += ",\n  ";
  otherAttributes += id;
  otherAttributes += " = ";
  if (type == STRING)
    otherAttributes += '"';
  otherAttributes += value;
  if (type == STRING)
    otherAttributes += '"';
}

const char* MacroOrString::getString ()
{
  const char* ret;

  switch (type)
    {
    case MACRO:
      return lookupMacro ((const char*) str);
    case STRING:
    case CATENATED:
      convertToCurrentFormat (str, representation);
      ret = (const char*) str;
      if (!ret || ret[0] == '\0')
	return 0;
      return ret;
    case UNDEFINED:
      break;
    default:
      barracuda_error ();
   };
  return 0;
}

const char* MacroOrString::getMacro ()
{
  const char* ret;

  switch (type)
    {
    case MACRO:
      return (const char*) str;
    case STRING:
    case CATENATED:
      convertToCurrentFormat (str, representation);
      ret = (const char*) str;
      if (!ret || ret[0] == '\0')
	return 0;
      return ret;
    case UNDEFINED:
      break;
    default:
      barracuda_error ();
   };
  return 0;
}

bool MacroOrString::convertTex = 0;

void MacroOrString::setConvertTex (bool b)
{
  MacroOrString::convertTex = b;
}

void MacroOrString::setStr (char* s)
{
  /* Convert incoming string and set str */
  if ((representation = MacroOrString::convertTex))
    str = texToString (s);
  else
    str = stringToTex (s);
}
