#include <qstring.h>
#include <stdio.h>
#include <ctype.h>


int myCharCmp(char c1, char c2);
int myStringCmp(const char *s1, const char *s2);


QString
getFirstName(const QString &s, QString *rest)
{
    int index = s.find(" and ", 0, FALSE);

    *rest = (s.right(s.length() - index - 4)).stripWhiteSpace();
    return (s.left(index)).stripWhiteSpace();
}


QString
wordBeforeHasSmallCase(int index, const QString &s, QString *rest)
{
    const char *s2 = (const char *) s;
    
    while (isspace(s2[index]) && index >= 0)
	index--;
    while (!isspace(s2[index]) && index >= 0)
	index--;
    if (index != 0)
	index++;
    
    if (islower(s2[index]))
    {
	*rest = (s.left(index)).stripWhiteSpace();
	return s.right(s.length()-index).stripWhiteSpace();
    }
    else
	return "";
}

    


QString 
getLastName(const QString &s, QString *rest)
{
    int index = s.findRev(" ");
    QString tmp = wordBeforeHasSmallCase(index, s, rest);
    
    if (tmp.isEmpty())
    {
	*rest = (s.left(index)).stripWhiteSpace();
	return (s.right(s.length()-index)).stripWhiteSpace();
    }
    else
	return tmp;
}
	


QString 
getPartBeforeComma(const QString &s, QString *rest)
{
    int index = s.find(",");
    *rest = (s.right(s.length()-index-1)).stripWhiteSpace();
    
    return (s.left(index)).stripWhiteSpace();
}


QString 
normalizeName(QString name)
{
    name = name.stripWhiteSpace();
    if (name.contains(","))
    {
	QString rest;
	QString first_part = getPartBeforeComma(name, &rest);
	return first_part + rest;
    }

    if (name.contains(" "))
    {
	QString rest;
    
	QString first_part = getLastName(name, &rest).stripWhiteSpace();
	return first_part + rest;
    }
    return name;
}

    
QString 
normalizeAuthorString(QString author)
{
    QString result;
    
    author = author.stripWhiteSpace();
    if (author.isEmpty())
	return author;

    if (author.contains(" and ", FALSE))
    {
	QString rest;
	QString firstPart = normalizeName(getFirstName(author, &rest));
	return firstPart + normalizeAuthorString(rest);
    }
       
    QString name = normalizeName(author);
    return name;
}
	

bool
hasBraces(QString &s)
{
    return (s.contains("{") && s.contains("}"));
}

QString 
stripFromBraces(QString s)
{
    int c = 0;
    while (hasBraces(s) && c++ < 255)
    {
	int index = s.find("{");
	
	QString first = (s.left(index)).stripWhiteSpace();
	index = s.find("}");
	QString last = (s.right(s.length()-index-1)).stripWhiteSpace(); 
	s = first + last;
    }

    if (c == 255)
    {
	fprintf(stderr, "ERROR: stripFromBraces sorry\n");
	exit(1);
    }

    return s;
}


int
authorCompare(const char *author1, const char *author2)
{
    QString a1 = author1;
    QString a2 = author2;

    if (hasBraces(a1)) a1 = stripFromBraces(a1);
    if (hasBraces(a2)) a2 = stripFromBraces(a2);

    a1 = normalizeAuthorString(author1);
    a2 = normalizeAuthorString(author2);

    return myStringCmp(a1, a2);
}






