/*
 * Bib-file parser.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

%{

/* C++ declarations */
#include <barracuda.h>
#include <keywords.h>
#include <stdio.h>
#include <db.h>
#include <stdlib.h>
#include <qstring.h>
#include <qmsgbox.h>
#include <Interface.h>
#include <MBMessageBox.h>

extern Interface* theInterface;
extern Options* theOptions;

void biberror (char* s);

bool lowerIdentifiers = TRUE;
char parserError[PARSER_ERROR_LEN+1];

typedef struct _semanticValue
{
  int type;
  char *str;
  int line;
} semanticValue;

#define YYSTYPE semanticValue

int biblex (YYSTYPE* yylval);

Reference* buffer;
extern int currentLine;
extern int scanString;
extern bool findAt;
Database* currentDb;

char* id2;
semanticValue debugWatch;
int referencesAdded;

%}

/* Bison declarations */

%token ATSTRING END_OF_FILE
%token Identifier StringEnd Number
%start BibFile
%pure_parser

%%

/* Grammar */
BibFile: OptMacroPart OptReferencePart END_OF_FILE { YYACCEPT; }
       | error END_OF_FILE
{
  sprintf (parserError, "Parse error. Unexpected end of file.");
  YYABORT;
}
       ;

OptMacroPart: /* empty */
            | OptMacroPart Macro
            ;

OptReferencePart: /* empty */
                | OptReferencePart Reference
                ;

Attributes: Attribute
          | Attributes ',' Attribute
          ;

String: '{' { scanString = 2; } StringEnd { $$ = $3; scanString = 0; }
      | '"' { scanString = 1; } StringEnd { $$ = $3; scanString = 0; }
      ;

AttributeValue: StringOrMacro
              | AttributeValue '#' StringOrMacro
{
  char* oldString = $1.str;

  /* The cocatenation is explicitely showed to the user */

  /* $1 and or $3 may be a strings that has to be surrounded by ":s */

  $$ = $1;
  $$.str = (char*) malloc (strlen ($$.str) + 
			   /* strlen (" # ")*/ 3 +
			   (($1.type == STRING) ? 2 : 0) +
			   (($3.type == STRING) ? 2 : 0) +
			   strlen ($3.str) + 1);

  $$.str[0] = '\0';

  if ($1.type == STRING)
    strcat ($$.str, "\"");
  strcat ($$.str, oldString);
  if ($1.type == STRING)
    strcat ($$.str, "\"");

  strcat ($$.str, " # ");

  if ($3.type == STRING)
    strcat ($$.str, "\"");
  strcat ($$.str, $3.str);
  if ($3.type == STRING)
    strcat ($$.str, "\"");

  free (oldString);
  free ($3.str);

  $$.type = CATENATED;
}
              ;

StringOrMacro: String
{ 
  debugWatch = $1;

  $$ = $1;
  $$.type = STRING; 
}
             | Identifier 
{ 
  debugWatch = $1;

  $$ = $1;

  /* Check that the identifier is a defined macro */
  if (!lookupMacro ($1.str))
    {
      sprintf (parserError, "Line %d: Undefined macro %s.",
	       $1.line, $1.str);
      delete buffer;
      YYABORT;			/* Barf */
    }

  $$.type = MACRO;
}
             | Number
{
  $$ = $1;
  $$.type = STRING; 
}
             ;

Macro : ATSTRING '{' Identifier '=' AttributeValue '}'
{
  if (!enterMacro ($3.str, $5.str, (text_type_t)$4.type))
    {
      sprintf (parserError, "Line %d: Redefinition of macro %s.",
	       $3.line, $3.str);
      free ($3.str);
      free ($5.str);
      YYABORT;			/* Barf */
    }
  free ($3.str);
  findAt = 1;
}
      | ATSTRING error '}'
{
  sprintf (parserError, "Line %d: Parse error before '}'.", $3.line);
  YYABORT;
}
      ;

StartEntry : '{'
           | '('
           ;

EndEntry : '}'
         | ')'
         ;

OptComma : /* empty */
         | ','
         ;

ReferenceID : Identifier | Number ;

Reference: Identifier
{
  /* Allocate buffer */
  if ($1.type < FIRST_REFERENCE || $1.type > LAST_REFERENCE)
    {
      sprintf (parserError, "Line %d: Illegal reference type.", $1.line);
      free ($1.str);
      YYABORT;			/* Barf */
    }

  buffer = new Reference($1.type);
  free ($1.str);
}
  StartEntry { lowerIdentifiers = FALSE; } ReferenceID
{ 
  lowerIdentifiers = TRUE;
} 
  ',' Attributes OptComma EndEntry
{
  id2 = $5.str;

  buffer->id = $5.str;
  free ($5.str);

  /* Insert reference in database */
  if (!currentDb->addReference (buffer))
    {
      parserError[0] = '\0';
      YYABORT;			/* Barf */
    }
  referencesAdded++;
  if (!(referencesAdded % 4))
    {
      QString m;
      m.sprintf ("Loaded %d references.", referencesAdded);
      theInterface->setInfoField (m);
    }
  findAt = 1;
}
         | Identifier error EndEntry
{
  sprintf (parserError, "Line %d: Parse error before [})].", $3.line);
  delete buffer;
  YYABORT;
}
         ;

Attribute: Identifier '=' AttributeValue
{
  /* Set corresponding attribute */
  switch (buffer->setField ($1.type, $3.str, (text_type_t) $3.type, 0))
    {
    case REDEFINITION:
      sprintf (parserError, "Line %d: Redefinition of attribute '%s'.",
	       $1.line, $1.str);
      free ($1.str);
      free ($3.str);
      delete buffer;
      YYABORT;

    case INVALID_FIELD:
      if (!theOptions->getPreference (NO_SILLY_WARN))
	{
	  QString msg;

	  msg.sprintf ("Line %d: '%s' is not a valid attribute for '%s'.",
		       $1.line, $1.str, keywords[buffer->getType ()]);

	  switch (MBMessageBox::query ("Warning", msg, 3, 
				       "Keep Attribute", "Ignore", "Abort"))
	    {
	    case 0:		/* Keep Attribute */
	      /* Copy attribute to otherAttributes */
	      buffer->addOtherAttribute ($1.str, $3.str, (text_type_t)$3.type);
	      break;

	    case 1:		/* Ignore */
	      break;

	    case 2:		/* Abort */
	      sprintf (parserError, "Line %d: Database open aborted.",
		       $1.line);
	      free ($1.str);
	      free ($3.str);
	      delete buffer;
	      YYABORT;
	    }
	}
      else			/* Keep attribute */
	{
	  /* Copy attribute to otherAttributes */
	  buffer->addOtherAttribute ($1.str, $3.str, (text_type_t) $3.type);
	}
      free ($1.str);
      free ($3.str);

      break;

    case OK:
      free ($1.str);
      free ($3.str);
      break;

    default:
      barracuda_error ();
      break;
    };
}
         ;

%%

/* C++ code */
void biberror (char*)
{
  sprintf (parserError, "Parse error.");
}
