/*
 * Conversion functions.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include <barracuda.h>
#include <qstring.h>
#include <qfont.h>

/* texToString
 *
 * Args: string in tex format
 * => string in displayable format
 */
QString texToString (const char* str)
{
  QString ret("");

  int index = 0;

  static struct {
    char accent;
    char letter;
    unsigned char isoLatin1;
  } accentLetters[] = {
    {'\'', 'A', 193},
    {'\'', 'E', 201},
    {'\'', 'I', 205},
    {'\'', 'O', 211},
    {'\'', 'U', 218},
    {'\'', 'Y', 221},
    {'\'', 'a', 225},
    {'\'', 'e', 233},
    {'\'', 'i', 237},
    {'\'', 'o', 243},
    {'\'', 'u', 250},
    {'\'', 'y', 253},
    {'`', 'A', 192},
    {'`', 'E', 200},
    {'`', 'I', 204},
    {'`', 'O', 210},
    {'`', 'U', 217},
    {'`', 'a', 224},
    {'`', 'e', 232},
    {'`', 'i', 236},
    {'`', 'o', 242},
    {'`', 'u', 249},
    {'^', 'A', 194},
    {'^', 'E', 202},
    {'^', 'I', 206},
    {'^', 'O', 212},
    {'^', 'U', 219},
    {'^', 'a', 226},
    {'^', 'e', 234},
    {'^', 'i', 238},
    {'^', 'o', 244},
    {'^', 'u', 251},
    {'"', 'A', 196},
    {'"', 'E', 203},
    {'"', 'I', 207},
    {'"', 'O', 214},
    {'"', 'U', 220},
    {'"', 'a', 228},
    {'"', 'e', 235},
    {'"', 'i', 239},
    {'"', 'o', 246},
    {'"', 'u', 252},
    {'~', 'A', 195},
    {'~', 'N', 209},
    {'~', 'O', 213},
    {'~', 'a', 227},
    {'~', 'n', 241},
    {'~', 'o', 245},
    {0, 0, 0}};

  /* Convert symbols in table 3.1 and table 3.2 of Lamport */
  /* Symbols not handled are \S \P \copyright \pounds ?` !` */

  int charIndex;

  while (str[index])
    if (str[index] == '{' && str[index+1] == '\\')
      {
	/* {\ <something> } */
	switch (str[index+2])
	  {
	  case '\'':
	  case '`':
	  case '^':
	  case '"':
	  case '~':
	    if (str[index+3] == '{'
		&& str[index+4] &&
		str[index+5] == '}' &&
		str[index+6] == '}')
	      /* {\'{c}} */
	      charIndex = 1;
	    else if (str[index+3] && str[index+4] == '}')
	      /* {\'c} */
	      charIndex = 0;
	    else
	      {
		ret += str[index++];
		break;
	      }
	    int i;
	    for (i = 0; accentLetters[i].accent; i++)
	      if (str[index+2] == accentLetters[i].accent &&
		  str[index+3+charIndex] == accentLetters[i].letter)
		{
		  ret += accentLetters[i].isoLatin1;
		  break;
		}
	    if (accentLetters[i].accent)
	      index += 5 + 2*charIndex;
	    else
	      ret += str[index++];
	    break;

	  case 'c':
	    /* {\c{c}} */
	    if (str[index+3] == '{' && str[index+4] && str[index+5] == '}' &&
		(str[index+4] == 'C' || str[index+4] == 'c'))
	      {
		if (str[index+4] == 'C')
		  ret += (char) 199;
		else if (str[index+4] == 'c')
		  ret += (char) 231;
		index += 7;
	      }
	    else
	      ret += str[index++];
	    break;
	    
	  case 'o':
	    if (str[index+3] == '}')
	      {
		ret += (char) 248;
		index += 5;
	      }
	    else
	      ret += str[index++];
	    break;
	      
	  case 'O':
	    if (str[index+3] == '}')
	      {
		ret += (char) 216;
		index += 5;
	      }
	    else
	      ret += str[index++];
	    break;

	  case 'a':
	    if (str[index+3] == 'e' && str[index+4] == '}')
	      {
		ret += (char) 230;
		index += 5;
	      }
	    else if (str[index+3] == 'a' && str[index+4] == '}')
	      {
		ret += (char) 229;
		index += 5;
	      }
	    else
	      ret += str[index++];
	    break;

	  case 'A':
	    if (str[index+3] == 'E' && str[index+4] == '}')
	      {
		ret += (char) 198;
		index += 5;
	      }
	    else if (str[index+3] == 'A' && str[index+4] == '}')
	      {
		ret += (char) 197;
		index += 5;
	      }
	    else
	      ret += str[index++];
	    break;

	  case 's':
	    if (str[index+3] == 's' && str[index+4] == '}')
	      {
		ret += (char) 223;
		index += 5;
	      }
	    else
	      ret += str[index++];
	    break;

	  default:
	    ret += str[index++];
	  }
      }
    else
      {
	ret += str[index++];
      }

  return ret;
}

/* stringToTex (hopefully not needed)
 *
 * Args: string in displayable
 * => string in tex format
 */
QString stringToTex (const char* str)
{
  QString ret;

  ret = str;

  return ret;
}

/* convertToCurrentFormat
 *
 * Args: string to convert and its representation
 * => side effect only
 */
void convertToCurrentFormat (QString& str, bool& representation)
{
  if (MacroOrString::convertTex && !representation)
    str = stringToTex ((const char*) str);
  else if (!MacroOrString::convertTex && representation)
    str = texToString ((const char*) str);
  representation = MacroOrString::convertTex;
}

/* escapeSpecialSymbols
 *
 * Args: a string and a symbol
 */
void escapeSpecialSymbols (QString& str, const char c)
{
  switch (c)
    {
    case '#':
    case '$':
    case '%':
    case '&':
    case '_':
    case '{':
    case '}':
      str += '\\';
      str += c;
      break;

    case '~':
      str += "$\\tilde{}$";
      break;

    case '^':
      str += "$\\hat{}$";
      break;

    case '\\':
      str += "$\\backslash{}$";
      break;

    default:
      str += c;
    };
}

/* Font handling */
/* Font format:
   family-pointSize-(light|normal|demibold|bold|black)-(italic|*)-(strikeout|*)
   */
QString externalFontRepresentation (QFont font)
{
  QString str;
  char tmpSpace[20];

  str = font.family ();
  str += '-';
  sprintf (tmpSpace, "%d", font.pointSize ());
  str += tmpSpace;
  str += '-';
  switch (font.weight ())
    {
    case (QFont::Light):
      str += "light";
      break;
    case (QFont::DemiBold):
      str += "demibold";
      break;
    case (QFont::Bold):
      str += "bold";
      break;
    case (QFont::Black):
      str += "black";
      break;
    default:
    case (QFont::Normal):
      str += "normal";
      break;
    }
  str += '-';
  str += (font.italic ()) ? "italic-" : "*-";
  str += (font.strikeOut ()) ? "strikeout-" : "*-";
  str += (font.underline ()) ? "underline" : "*";

  return str;
}

/* Font format:
   family-pointSize-(light|normal|demibold|bold|black)-(italic|*)-(strikeout|*)
   */
bool internalFontRepresentation (QFont& font, const char* str)
{
  QString qstr = str;
  int lastDash = 0, nextDash;

  /* Family */
  if ((nextDash = qstr.find ('-', lastDash)) == -1)
    return false;
  font.setFamily ((const char*) qstr.mid (lastDash, nextDash-lastDash));

  /* Pointsize */
  lastDash = nextDash+1;
  if ((nextDash = qstr.find ('-', lastDash)) == -1)
    return false;
  font.setPointSize (atoi ((const char*) qstr.mid (lastDash, nextDash-lastDash)));

  /* Weight */
  lastDash = nextDash+1;
  if ((nextDash = qstr.find ('-', lastDash)) == -1)
    return false;
  if (qstr.mid (lastDash, nextDash-lastDash) == "light")
    font.setWeight (QFont::Light);
  else if (qstr.mid (lastDash, nextDash-lastDash) == "demibold")
    font.setWeight (QFont::DemiBold);
  else if (qstr.mid (lastDash, nextDash-lastDash) == "bold")
    font.setWeight (QFont::Bold);
  else if (qstr.mid (lastDash, nextDash-lastDash) == "black")
    font.setWeight (QFont::Black);
  else if (qstr.mid (lastDash, nextDash-lastDash) == "normal")
    font.setWeight (QFont::Normal);
  else
    return false;

  /* Italic */
  lastDash = nextDash+1;
  if ((nextDash = qstr.find ('-', lastDash)) == -1)
    return false;
  font.setItalic (qstr.mid (lastDash, nextDash-lastDash) == "italic");

  /* StrikeOut */
  lastDash = nextDash+1;
  if ((nextDash = qstr.find ('-', lastDash)) == -1)
    return false;
  font.setStrikeOut (qstr.mid (lastDash, nextDash-lastDash) == "strikeout");

  /* Underline */
  lastDash = nextDash+1;
  if ((nextDash = qstr.find ('-', lastDash)) != -1)
    return false;
  font.setUnderline (qstr.mid (lastDash, qstr.length ()-1) == "underline");

  return true;
}
