/*
 * Font select dialog. (This was ripped from one of the Qt examples,
 *                      I hope no one minds :-)
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include "FontSelect.h"

FontSelect::FontSelect( QWidget *parent, const char *name)
    : QDialog( parent, name, 0 ), f( "Charter", 48, QFont::Bold )
{
    static const char *radios[] = {
    	"Light (25)", "Normal (50)", "DemiBold (63)",
	"Bold (75)", "Black (87)"
    };
    int i;

    fontInternal = new QWidget( this );
    fontInternal->setFont( f );
    fontInternal->hide();

    familyL    = new QLabel(	 this, "familyLabel" );
    sizeL      = new QLabel(	 this, "sizeLabel" );
    family     = new QLineEdit(	 this, "family" );
    sizeE      = new QLineEdit(	 this, "pointSize" );
    italic     = new QCheckBox(	 this, "italic" );
    underline  = new QCheckBox(	 this, "underline" );
    strikeOut  = new QCheckBox(	 this, "strikeOut" );
    apply      = new QPushButton( this, "apply" );
    sizeS      = new QScrollBar( QScrollBar::Horizontal, this,
			      "pointSizeScrollBar" );

    yOff = 35;
    familyL->setGeometry( 10, yOff + 10, 100,20 );
    familyL->setText( "Family :" );

    sizeL->setGeometry( 10, yOff + 40, 100, 20 );
    sizeL->setText( "Point size :" );

    family->setGeometry( 110, yOff + 10, 100, 20 );
    family->setText( "Times" );

    sizeE->setGeometry( 110, yOff + 40, 100, 20 );
    sizeE->setText( "12" );

    sizeS->setGeometry( 220, yOff + 40, 100, 20 );
    sizeS->setRange( 1, 100 );
    sizeS->setValue( 12 );
    sizeS->setTracking( FALSE );
    connect( sizeS, SIGNAL(valueChanged(int)), SLOT(newSize(int)) );
    connect( sizeS, SIGNAL(sliderMoved(int)),  SLOT(slidingSize(int)) );

    italic->setGeometry( 10, yOff + 70, 80, 20 );
    italic->setText( "Italic" );
    connect( italic, SIGNAL(clicked()), SLOT(newItalic()) );

    underline->setGeometry( 110, yOff + 70, 80, 20 );
    underline->setText( "Underline" );
    connect( underline, SIGNAL(clicked()), SLOT(newUnderline()) );

    strikeOut->setGeometry( 210, yOff + 70, 80, 20 );
    strikeOut->setText( "StrikeOut" );
    connect( strikeOut, SIGNAL(clicked()), SLOT(newStrikeOut()) );

    apply->setGeometry( 235, yOff + 10, 70, 20);
    apply->setText( "APPLY" );
    apply->setDefault( TRUE );
    connect( apply, SIGNAL(clicked()), SLOT(doApply()) );

    weight = new QButtonGroup( "Weight", this, "weightGroupBox" );
    weight->setGeometry( 10, yOff + 100, 120, 120 );
    connect( weight, SIGNAL(clicked(int)), SLOT(newWeight(int)) );
    QString wname;
    for( i = 0 ; i < 5 ; i++ ) {
	wname.sprintf("radioButton %i",i);
	rb[i] = new QRadioButton( weight, wname );
	rb[i]->setGeometry( 10, 15+i*20 , 95, 20 );
	rb[i]->setText( radios[i] );
    }
    rb[3]->setChecked( TRUE );

    static const char *familys[] = {
	"Charter", "Clean", "Courier", "Fixed",
	"Gothic", "Helvetica", "Lucida", "Lucidabright",
	"Lucidatypewriter", "Mincho",
	"New century Schoolbook",
	"Symbol", "Terminal", "Times", "Utopia", "Arial", 0
    };

    static const char *pSizes[]  = {
	"8", "10", "12", "14", "18", "24", "36", "48", "72", "96", 0
    };

    QPopupMenu *family = new QPopupMenu;
    const char **tmp;
    tmp = familys;
    while( *tmp )
	family->insertItem( *tmp++ );

    QPopupMenu *pSize = new QPopupMenu;
    tmp = pSizes;
    while( *tmp )
	pSize->insertItem( *tmp++ );

    menu = new QMenuBar( this );
    menu->move( 0, 0 );
    menu->resize( 350, 30 );
    menu->insertItem( "Family", family );
    menu->insertItem( "Point size", pSize );

    connect( family, SIGNAL(activated(int)), SLOT(familyActivated(int)) );
    connect( pSize, SIGNAL(activated(int)), SLOT(pSizeActivated(int)) );

    static const char *mLabelStr[] = {
	"Family:", "Point size:", "Weight:", "Italic:"
    };

    mGroup = new QButtonGroup( this, "metricsGroupBox" );
    mGroup->setTitle( "Actual font" );
    mGroup->setGeometry(140, yOff + 100, 230, 100);
    for( i = 0 ; i < 4 ; i++ ) {
	wname.sprintf("MetricsLabel[%i][%i]",i,0);
	metrics[i][0] = new QLabel( mGroup, wname);
	metrics[i][0]->setGeometry(10, 15 + 20*i, 70, 20);
	metrics[i][0]->setText( mLabelStr[i] );

	wname.sprintf("MetricsLabel[%i][%i]",i,1);
	metrics[i][1] = new QLabel( mGroup, wname);
	metrics[i][1]->setGeometry(90, 15 + 20*i, 135, 20);
    }
    updateMetrics();
}

void FontSelect::newStrikeOut()
{
    f.setStrikeOut( strikeOut->isChecked() );
    doFont();
}

void FontSelect::doFont()
{
    QFont xyz = f;
    xyz.setPointSize( f.pointSize()+1 );
    xyz.setPointSize( f.pointSize() );
    fontInternal->setFont( xyz );
    updateMetrics();
}

void FontSelect::newUnderline()
{
    f.setUnderline( underline->isChecked() );
    doFont();
}

void FontSelect::newItalic()
{
    f.setItalic( italic->isChecked() );
    doFont();
}

void FontSelect::newFamily()
{
    f.setFamily( family->text() );
    doFont();
}

void FontSelect::newWeight( int id )
{
    switch( id ) {
	case 0 :
	    f.setWeight( QFont::Light );
	    break;
	case 1 :
	    f.setWeight( QFont::Normal );
	    break;
	case 2 :
	    f.setWeight( QFont::DemiBold );
	    break;
	case 3 :
	    f.setWeight( QFont::Bold );
	    break;
	case 4 :
	    f.setWeight( QFont::Black );
	    break;
	default:
	    return;
    }
    doFont();
}

void FontSelect::newSize( int value )
{
    QString tmp;
    tmp.sprintf("%i", value);
    sizeE->setText( tmp );
    f.setPointSize( value );
    doFont();
}

void FontSelect::slidingSize( int value )
{
    QString tmp;

    tmp.sprintf("%i", value);
    sizeE->setText( tmp );
}

void FontSelect::doApply()
{
    int sz = atoi( sizeE->text() );
    if ( sz > 100) {
	sizeS->blockSignals( TRUE );
	sizeS->setValue( 100 );
	sizeS->blockSignals( FALSE );
	f.setPointSize( sz );
    } else {
	sizeS->setValue( atoi( sizeE->text() ) );
    }
    f.setFamily( family->text() );
    doFont();

    emit newFont( fontInternal->font() );
    hide ();
}

void FontSelect::familyActivated( int id )
{
    family->setText( ((QPopupMenu*)sender())->text(id) );
    newFamily();
}

void FontSelect::pSizeActivated( int id )
{
    int value = atoi( ( (QPopupMenu*)sender())->text( id ) );
    sizeS->blockSignals( TRUE );
    sizeS->setValue( value );
    sizeS->blockSignals( FALSE );
    newSize( value );
}

void FontSelect::showFont (const QFont newF)
{
    int i;
    char val[100];

    family->setText (newF.family ());
    newFamily ();

    newSize (newF.pointSize ());
    sizeS->setValue (newF.pointSize ());

    italic->setChecked(newF.italic());
    newItalic ();

    underline->setChecked (newF.underline());
    newUnderline ();

    strikeOut->setChecked (newF.strikeOut());
    newStrikeOut ();

    for (i = 0; i < 5; i++)
	rb[i]->setChecked(FALSE);
    if (newF.weight() == QFont::Light)
	i = 0;
    else if (newF.weight() == QFont::Normal)
	i = 1;
    else if (newF.weight() == QFont::DemiBold)
	i = 2;
    else if (newF.weight() == QFont::Bold)
	i = 3;
    else if (newF.weight() == QFont::Black)
	i = 4;

    rb[i]->setChecked (TRUE);
    newWeight (i);
}

void FontSelect::updateMetrics()
{
    QFontInfo fi = fontInternal->fontInfo();
    metrics[0][1]->setText( fi.family() );
    metrics[1][1]->setNum( fi.pointSize() );
    metrics[2][1]->setNum( fi.weight() );
    metrics[3][1]->setNum( fi.italic() );
}
