#include <qpopmenu.h>
#include <qkeycode.h>
#include <qapp.h>
#include <qpixmap.h>
#include "MenuBarData.h"

char *menuTexts[] = {
    "New",
    "Open",
    "Database file",
    "Macro file", 
    "Save",
    "Save As...",
    "Save Macros...",
    "Print...",
    "Exit",
    "Insert Record",
    "Delete Record",
    "Edit Record", 
    "Edit Macros", 
    "Preferences...",
    "Simple search...", 
    "Keybindings...",
    "View Types...",
    "Save Options", 
    "Search", 
    "Sort", 
    "On-line Help",
    "About...",
    "Article",
    "InProceedings",
    "InCollection",
    "InBook",
    "Book",
    "Booklet", 
    "PhDThesis", 
    "MastersThesis", 
    "TechReport", 
    "Manual", 
    "Unpublished", 
    "Misc", 
    "Proceedings"
};
    

/* MenuBarData
 *
 * Args: parent, name
 * => ???
 */

MenuBarData::MenuBarData()
{
    open = new QPopupMenu;
    CHECK_PTR (open);
    open->insertItem ("Database file", OPEN_DB_MENU_ID);
    open->insertItem ("Macro file", OPEN_MACRO_MENU_ID);

    file = new QPopupMenu;
    CHECK_PTR (file);
    file->insertItem ("New", NEW_MENU_ID);
    file->insertItem ("Open", open, OPEN_MENU_ID);
    file->insertItem ("Save", SAVE_MENU_ID); 
    file->insertItem ("Save As...", SAVE_AS_MENU_ID); 
    file->insertItem ("Save Macros...", SAVE_MACROS_MENU_ID); 
    file->insertSeparator ();
    file->insertItem ("Print...", PRINT_MENU_ID);
    file->insertSeparator ();
    file->insertItem ("Exit", EXIT_MENU_ID);

    rec_ins = new QPopupMenu;
    CHECK_PTR (rec_ins);
    rec_ins->insertItem ("Article", ARTICLE_MENU_ID);
    rec_ins->insertItem ("InProceedings", INPROCEEDINGS_MENU_ID);
    rec_ins->insertItem ("InCollection", INCOLLECTION_MENU_ID);
    rec_ins->insertItem ("InBook", INBOOK_MENU_ID);
    rec_ins->insertItem ("Book", BOOK_MENU_ID);
    rec_ins->insertItem ("Booklet", BOOKLET_MENU_ID);
    rec_ins->insertItem ("PhDThesis", PHDTHESIS_MENU_ID);
    rec_ins->insertItem ("MastersThesis", MASTERSTHESIS_MENU_ID);
    rec_ins->insertItem ("TechReport", TECHREPORT_MENU_ID);
    rec_ins->insertItem ("Manual", MANUAL_MENU_ID);
    rec_ins->insertItem ("Unpublished", UNPUBLISHED_MENU_ID);
    rec_ins->insertItem ("Misc", MISC_MENU_ID);
    rec_ins->insertItem ("Proceedings", PROCEEDINGS_MENU_ID);
    
    edit = new QPopupMenu;
    CHECK_PTR (edit);
    edit->insertItem ("Insert Record", rec_ins, INSERT_RECORD_MENU_ID);
    edit->insertItem ("Delete Record", DELETE_RECORD_MENU_ID);
    edit->insertItem ("Edit Record", EDIT_RECORD_MENU_ID);
    edit->insertSeparator ();    
    edit->insertItem ("Edit Macros", EDIT_MACRO_MENU_ID);

    option = new QPopupMenu;
    CHECK_PTR (option);
    option->insertItem ("Preferences...", PREFERENCES_MENU_ID);
    option->insertItem ("Keybindings...", KEYBINDINGS_MENU_ID);
    option->insertItem ("View Types...", VIEW_TYPES_MENU_ID);
    option->insertSeparator ();    
    option->insertItem ("Save Options", SAVE_OPTIONS_MENU_ID);
    
    tools = new QPopupMenu;
    CHECK_PTR (tools);
    tools->insertItem ("Search", SEARCH_MENU_ID);
    tools->insertItem ("Simple search...", SIMPLE_SEARCH_MENU_ID);
    tools->insertItem ("Sort", SORT_MENU_ID);
    
    help = new QPopupMenu;
    CHECK_PTR (help);
    help->insertItem (menuTexts[SPAWN_BROWSER_MENU_ID], SPAWN_BROWSER_MENU_ID);
    help->insertSeparator ();    
    help->insertItem ("About...", ABOUT_MENU_ID);

}


/* findId
 *
 * Args: id (An id for a menu item)
 * => The menu for the given id
 */

QPopupMenu *MenuBarData::findId (int id)
{
    switch (id)
    {
    case NEW_MENU_ID:
    case OPEN_MENU_ID:
    case SAVE_MENU_ID:
    case SAVE_AS_MENU_ID:
    case SAVE_MACROS_MENU_ID:
    case PRINT_MENU_ID:
    case EXIT_MENU_ID:
	return file;

    case OPEN_DB_MENU_ID:
    case OPEN_MACRO_MENU_ID:
	return open;

    case ARTICLE_MENU_ID:
    case INPROCEEDINGS_MENU_ID:
    case INCOLLECTION_MENU_ID:
    case INBOOK_MENU_ID:
    case BOOK_MENU_ID:
    case BOOKLET_MENU_ID:
    case PHDTHESIS_MENU_ID:
    case MASTERSTHESIS_MENU_ID:
    case TECHREPORT_MENU_ID:
    case MANUAL_MENU_ID:
    case UNPUBLISHED_MENU_ID:
    case MISC_MENU_ID:
    case PROCEEDINGS_MENU_ID:
	return rec_ins;

    case DELETE_RECORD_MENU_ID:
    case EDIT_RECORD_MENU_ID:
    case EDIT_MACRO_MENU_ID:
	return edit;
    
    case PREFERENCES_MENU_ID:
    case KEYBINDINGS_MENU_ID:
    case VIEW_TYPES_MENU_ID:
    case SAVE_OPTIONS_MENU_ID:
	return option;
    
    case SEARCH_MENU_ID:
    case SIMPLE_SEARCH_MENU_ID:
    case SORT_MENU_ID:
	return tools;

    case SPAWN_BROWSER_MENU_ID:
    case ABOUT_MENU_ID:
	return help;

    default:
	return NULL;
    }
}


/* getText
 *
 * Args: id
 * => Text for the given id
 */

char *MenuBarData::getText (int id)
{
    if (id >= 0 && id <= MAX_MENU_ID)
	return menuTexts[id];

    return NULL;
}
