/*
 * Preferences dialog.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include "Preferences.h"
#include <qpixmap.h>

#define Inherited QDialog

Preferences::Preferences(QWidget* parent, const char* name)
	: Inherited( parent, name, TRUE), PreferencesData( this )
{
	setCaption( "Preferences" );
	delimiterBox->insertItem("None");

	linepm = new QPixmap();
	dottedlinepm = new QPixmap();
	dashlinepm = new QPixmap();
	dashdotlinepm = new QPixmap();
	dashdotdotlinepm = new QPixmap();
	
	
	QString path = getenv("BARRACUDAHOME");
	QString def_path = BARRACUDA_PATH;
	if (linepm->load(path + "/bitmaps" "/line.bmp"))
	    delimiterBox->insertItem(*linepm);
	else
	    if (linepm->load(def_path + "/bitmaps" "/line.bmp"))
		delimiterBox->insertItem(*linepm);

	if (dashlinepm->load(path + "/bitmaps" "/dashline.bmp"))
	    delimiterBox->insertItem(*dashlinepm);
	else
	    if (dashlinepm->load(def_path + "/bitmaps" "/dashline.bmp"))
		delimiterBox->insertItem(*dashlinepm);

	if (dottedlinepm->load(path + "/bitmaps" "/dottedline.bmp"))
	    delimiterBox->insertItem(*dottedlinepm);
	else
	    if (dottedlinepm->load(def_path + "/bitmaps" "/dottedline.bmp"))
		delimiterBox->insertItem(*dottedlinepm);

	if (dashdotlinepm->load(path + "/bitmaps" "/dashdotline.bmp"))
	    delimiterBox->insertItem(*dashdotlinepm);
	else
	    if (dashdotlinepm->load(def_path + "/bitmaps" "/dashdotline.bmp"))
		delimiterBox->insertItem(*dashdotlinepm);

	if (dashdotdotlinepm->load(path + "/bitmaps" "/dashdotdotline.bmp"))
	    delimiterBox->insertItem(*dashdotdotlinepm);
	else
	    if (dashdotdotlinepm->load(def_path + "/bitmaps" 
				       "/dashdotdotline.bmp"))
		delimiterBox->insertItem(*dashdotdotlinepm);
	
	connect(okButton, SIGNAL(clicked()), this, SLOT(ok()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
	connect(defaultButton, SIGNAL(clicked()), this, SLOT(setDefault()));

	connect(caseSenseButton, SIGNAL(clicked()), this, 
		SLOT(caseSense()));
	connect(expandMacrosButton, SIGNAL(clicked()), this, 
		SLOT(expandMacros()));
	connect(lacyUpdateButton, SIGNAL(clicked()), this, 
		SLOT(lacyUpdate()));
	connect(saveUnusedButton, SIGNAL(clicked()), this, 
		SLOT(saveUnused()));
	connect(showStartUpButton, SIGNAL(clicked()), this, 
		SLOT(showStartUp()));
	connect(showWarningsButton, SIGNAL(clicked()), this, 
		SLOT(showWarnings()));
	connect(verboseButton, SIGNAL(clicked()), this, 
		SLOT(verbose()));
	connect(delimiterBox, SIGNAL(activated(int)), this, 
		SLOT(comboActive(int)));
}


Preferences::~Preferences()
{
}


void 
Preferences::setWidgets(Options *o)
{
    caseSenseButton->setChecked(o->getPreference(CASE_SEARCH));
    expandMacrosButton->setChecked(o->getPreference(EXPAND_MACRO));
    lacyUpdateButton->setChecked(o->getPreference(LAZY_COLUMN));
    saveUnusedButton->setChecked(o->getPreference(SAVE_UNUSED_MACROS));
    showStartUpButton->setChecked(!o->getPreference(NO_SILLY_STARTUP));
    showWarningsButton->setChecked(!o->getPreference(NO_SILLY_WARN));
    verboseButton->setChecked(o->getPreference(VERBOSE_PARSE));
    delimiterBox->setCurrentItem(o->getPreference(DEL_STYLE));
}


bool
Preferences::eShow(Options *o)
{
    expandMacrosButton->setChecked(o->getPreference(EXPAND_MACRO));
    lacyUpdateButton->setChecked(o->getPreference(LAZY_COLUMN));
    verboseButton->setChecked(o->getPreference(VERBOSE_PARSE));
    showWarningsButton->setChecked(!o->getPreference(NO_SILLY_WARN));
    showStartUpButton->setChecked(!o->getPreference(NO_SILLY_STARTUP));
    caseSenseButton->setChecked(o->getPreference(CASE_SEARCH));
    saveUnusedButton->setChecked(o->getPreference(SAVE_UNUSED_MACROS));

    oldopt = o;
    myopt = new Options(o->maxShortcuts());
    myopt->assign(o);
    this->setWidgets(myopt);
    this->show();

    return result();
}


/*
 * Preferences::ok()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::ok()
{
    oldopt->assign(myopt);
    delete myopt;
    this->done(TRUE);
}

/*
 * Preferences::cancel()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::cancel()
{
    delete myopt;
    this->done(FALSE);
}

/*
 * Preferences::setDefault()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::setDefault()
{
    myopt->setDefaultPref();
    setWidgets(myopt);
}


/*
 * Preferences::caseSense()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::caseSense()
{
    myopt->setPreference(CASE_SEARCH, caseSenseButton->isChecked());
}

/*
 * Preferences::expandMacros()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::expandMacros()
{
    myopt->setPreference(EXPAND_MACRO, expandMacrosButton->isChecked());
}

/*
 * Preferences::lacyUpdate()   (The 'c' is for historical reasons)
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::lacyUpdate()
{
    myopt->setPreference(LAZY_COLUMN, lacyUpdateButton->isChecked());
}

/*
 * Preferences::saveUnused()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::saveUnused()
{
    myopt->setPreference(SAVE_UNUSED_MACROS, saveUnusedButton->isChecked());
}

/*
 * Preferences::showStartUp()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::showStartUp()
{
    myopt->setPreference(NO_SILLY_STARTUP, !showStartUpButton->isChecked());
}

/*
 * Preferences::showWarnings()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::showWarnings()
{
    myopt->setPreference(NO_SILLY_WARN, !showWarningsButton->isChecked());
}

/*
 * Preferences::verbose()
 * Args: void
 * Returns: void
 * Results: 
 */
void
Preferences::verbose()
{
    myopt->setPreference(VERBOSE_PARSE, verboseButton->isChecked());
}

/*
 * Preferences::comboActive
 * Args: int index
 * Returns: void
 * Results: 
 */
void 
Preferences::comboActive(int index)
{
    myopt->setPreference(DEL_STYLE, index);
}





