/*
 * Sort dialog.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include "Sort.h"
#include <qmsgbox.h>
#include <stdlib.h>
#include <stdio.h>

#define Inherited QDialog


SortSpec::SortSpec()
{
    ordering = new int[LAST_FIELD-FIRST_FIELD+2];
    if (ordering == NULL)
	exit(1);
    makeDefault();
}

SortSpec::~SortSpec()
{
    delete [] ordering;
}


void
SortSpec::makeDefault()
{
    for (int i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
	ordering[i] = i;
    descending = FALSE;
}


Sort::Sort(QWidget* parent, const char* name)
	:
	Inherited( parent, name, TRUE ),
	SortData( this )
{
	setCaption( "Sort " );
	QComboBox *tmp[] = {cb1, cb2, cb3, cb4, cb5, cb6, cb7, cb8,
			    cb9, cb10, cb11, cb12, cb13, cb14, cb15,
			    cb16, cb17, cb18, cb19, cb20, cb21, cb22,
			    cb23, cb24, cb25};

	for (int i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
	    myCombo[i] = tmp[i];

	char *strTmp[] = {"Id", "Author", "Title", "Journal", "Year", "Key",
			  "Volume", "Number", "Month", "Pages", "Note", 
			  "Annote", "Booktitle", "Crossref", "Editor", 
			  "Series", "Organization", "Publisher", "Address", 
			  "Chapter", "Type", "Edition", "Howpublished", 
			  "School", "Institution"};
	for (int i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
	    for (int j = 0; j <= LAST_FIELD-FIRST_FIELD+1; j++)
		myCombo[j]->insertItem(strTmp[i]);
	    
	for (int i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
	    connect(myCombo[i], SIGNAL(activated(int)),
		    this, SLOT(comboPressed(int)));
	


	connect(okButton, SIGNAL(clicked()), this, SLOT(ok()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
	connect(descendButton, SIGNAL(clicked()), this, SLOT(descend()));
	connect(ascendButton, SIGNAL(clicked()), this, SLOT(ascend()));	
}


Sort::~Sort()
{
}


bool
Sort::eShow(SortSpec *s)
{
    int i;

    mySpec = s;
       
    for (i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
    {
	myCombo[i]->setCurrentItem(s->ordering[i]);
	choice[i] = mySpec->ordering[i];
    }
    descendButton->setChecked(mySpec->descending);
    ascendButton->setChecked(!mySpec->descending);
    show();
    return result();
}


bool
duplicated(int vect[], int n, int *x, int *y)
{
    for (int i = 0; i < n; i++)
	for (int j = 0; j < n; j++)
	    if (vect[i] == vect[j] && i != j)
	    {
		*x = j;
		*y = i;
		return TRUE;
	    }
    return FALSE;
}

void
Sort::ok()
{
    int tmp[LAST_FIELD-FIRST_FIELD+2];
        
    for (int i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
    	tmp[i] = myCombo[i]->currentItem();

    int i, j;
    if (duplicated(tmp, LAST_FIELD-FIRST_FIELD+2, &i, &j))
    {
	QString message;
	message.sprintf("Sort order %d and %d are identical.\n"
	                "That makes no sense", i+1, j+1);
	QMessageBox::message("Error", (const char *) message,
			     "Oops! I'm sorry");
	return;
    }
    
    for (i  = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
	mySpec->ordering[i] = tmp[i];
    mySpec->descending = descendButton->isOn();
    done(TRUE);    
}

void
Sort::cancel()
{
    done(FALSE);
}


void
Sort::descend()
{
    ascendButton->setChecked(FALSE);
}

void
Sort::ascend()
{
    descendButton->setChecked(FALSE);
}


int
findFree(int vect[], int n)
{
    int *tmp = (int *) malloc(n*sizeof(int));
    int i;

    for (i = 0; i < n; i++)
	tmp[i] = -1;

    for (i = 0; i < n; i++)
	tmp[vect[i]] = 1;

    for (i = 0; i < n; i++)
	if (tmp[i] == -1)
	{
	    free(tmp);
	    return i;
	}
    
    free(tmp);
    return -1;
}

void
Sort::comboPressed(int index)
{
    int tmp[LAST_FIELD-FIRST_FIELD+2];
    int i, j;

    for (i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
    	tmp[i] = myCombo[i]->currentItem();
    
    if (duplicated(tmp, LAST_FIELD-FIRST_FIELD+2, &i, &j))
    {
	if (tmp[i] != choice[i])
	    myCombo[j]->setCurrentItem(findFree(tmp, LAST_FIELD-FIRST_FIELD+2));
	else
	    myCombo[i]->setCurrentItem(findFree(tmp, LAST_FIELD-FIRST_FIELD+2));
    }
    
    for (i = 0; i <= LAST_FIELD-FIRST_FIELD+1; i++)
    	choice[i] = tmp[i];

    index++; // YAHOOO!! This will remove a warning!
}
    



