/*
 * Option-file scanner.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

%{

/* C++ declarations */

#include <ctype.h>
#include <string.h>

typedef struct _semanticValue
{
  int type;
  char *str;
  int number;
  int line;
} semanticValue;

#define YYSTYPE semanticValue

#define YY_NO_UNPUT 1

#define YY_DECL int optionlex (YYSTYPE* yylval)

#include "option.tab.h"
#include <keywords.h>

static int lookupIdentifier (char* id, int n);
int optionCurrentLine;

%}

FIRST          [A-Za-z]
OTHER          ([A-Za-z0-9.-]|\*)
STRINGLETTER   [^\"\}\{\n]

%option noyywrap

%%

<<EOF>>                { return END_OF_FILE; }

[\t ]                  { /* White space */ }

\n                     { optionCurrentLine++; }

[;:][^\n]*              { /* Comment */ }

[0-9]+                 {
  yylval->number = atoi (optiontext);
  yylval->line = optionCurrentLine;
  return Number;
                       }

"set-preference"       { yylval->line = optionCurrentLine;
                         return SetPreference; }

"set-column-font"      { yylval->line = optionCurrentLine;
                         return SetColumnFont; }

"set-shortcut"         { yylval->line = optionCurrentLine;
                         return SetShortcut; }

"show-reference"       { yylval->line = optionCurrentLine;
                         return SetField; }

"show-attribute"       { yylval->line = optionCurrentLine;
                         return SetType; }

"undef-shortcut"       { yylval->line = optionCurrentLine;
                         return UndefShortcut; }

"set-column-width"     { yylval->line = optionCurrentLine;
                         return SetColumnWidth; }

"set-column-index"     { yylval->line = optionCurrentLine;
                         return SetColumnIndex; }

"true"                 { yylval->line = optionCurrentLine;
                         yylval->type = 1; 
                         return Boolean; }

"false"                { yylval->line = optionCurrentLine;
                         yylval->type = 0;
                         return Boolean; }

")"                    { yylval->line = optionCurrentLine;
                         return RPAREN; }

{FIRST}{OTHER}*        { 
  if (!(yylval->type = lookupIdentifier (optiontext, optionleng)))
    yylval->str = strdup (optiontext);
  yylval->line = optionCurrentLine;
  return Identifier;
                       }

\"[^\"\n]+\"            {
  yylval->type = 0;
  yylval->line = optionCurrentLine;
  yylval->str = (char*) malloc (sizeof(char)*(optionleng-1));
  optiontext[optionleng-1] = '\0';
  strcpy (yylval->str, optiontext+1);

  return String;
                       }

.                      { return optiontext[0]; }

%%

/* C++ code */

/* lookupIdentifier
 *
 * Args: string, strlen (string)
 * => if keyword - Keyword number for string
 * => if command - Command number for string
 * => else 0
 */
static int lookupIdentifier (char* id, int n)
{
  int r;

  /* Find keyword */
  for (r = FIRST_PREF; r <= LAST_PREF; r++)
    if (!strncmp (optionLabels[r], id, n))
      return r;

  for (r = FIRST_REFERENCE; r <= LAST_FIELD+1; r++)
    if (!strncmp (keywords[r], id, n))
      return r;

  /* Find command */
  for (r = FIRST_COMMAND; r <= LAST_COMMAND; r++)
    if (!strncmp (commands[r], id, n))
      return -(r+1);

  /* Return ''not keyword'' */
  return 0;
}
