/* MLItem.cc
 *
 * Implements the MultiLabelItem class.
 * This class should be used by the MultiLabel widget.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */


#include "MLItem.h"

/* MultiLabelItem
 *
 * Constructor for the MultiLabelItem class.
 * Sets the alignment to AlignLeft | AlignVCenter,
 * Sets the margin to -1,
 * Sets visible to TRUE,
 * Disables autoResize;
 * Sets the font to the default font.
 */

MultiLabelItem::MultiLabelItem ()
{
    align = AlignLeft | AlignVCenter;
    marg = -1;
    vis = TRUE;
    f = QFont::defaultFont ();
}

MultiLabelItem::~MultiLabelItem ()
{
}


/* drawContents
 *
 * Draws the contents of the item within the given rectangle,
 * using the given painter. The painter should be open when it
 * gets here and will not be closed.
 */ 
   
void MultiLabelItem::drawContents (
    QRect clipRect,
    const QColor color,
    QPainter *p)
{
    int m = margin();
    int align = alignment();

    p->setFont (font());
    if (m < 0)
    {
	if (clipRect.width() > 0)
	    m = p->fontMetrics().width("x") / 2;
	else
	    m = 0;
    }
    if (align & AlignLeft)
	clipRect.setLeft (clipRect.left() + m);
    if (align & AlignRight)
	clipRect.setRight (clipRect.right() - m);
    if (align & AlignTop)
	clipRect.setTop (clipRect.top() + m);
    if (align & AlignBottom)
	clipRect.setBottom (clipRect.bottom() - m);
    p->setPen (color);
    p->drawText (clipRect, align, label());
} 

